@@ -1,7 +1,7 @@
 package Complete::Getopt::Long;
 
-our $DATE = '2014-12-25'; # DATE
-our $VERSION = '0.21'; # VERSION
+our $DATE = '2014-12-26'; # DATE
+our $VERSION = '0.22'; # VERSION
 
 use 5.010001;
 use strict;
@@ -22,11 +22,11 @@ sub _default_completion {
     my %args = @_;
     my $word = $args{word} // '';
 
-    $log->tracef('entering default completion');
+    $log->tracef('[comp][compgl] entering default completion routine');
 
     # try completing '$...' with shell variables
     if ($word =~ /\A\$/) {
-        $log->tracef('completing shell variable');
+        $log->tracef('[comp][compgl] completing shell variable');
         require Complete::Util;
         {
             my $compres = Complete::Util::complete_env(
@@ -39,7 +39,7 @@ sub _default_completion {
 
     # try completing '~foo' with user dir (appending / if user's home exists)
     if ($word =~ m!\A~([^/]*)\z!) {
-        $log->tracef("completing userdir");
+        $log->tracef("[comp][compgl] completing userdir, user=%s", $1);
         {
             eval { require Unix::Passwd::File };
             last if $@;
@@ -60,7 +60,7 @@ sub _default_completion {
     # expand ~foo (this is supported by complete_file(), so we just give it off
     # to the routine)
     if ($word =~ m!\A(~[^/]*)/!) {
-        $log->tracef("completing file");
+        $log->tracef("[comp][compgl] completing file, path=<%s>", $word);
         return {words=>Complete::Util::complete_file(word=>$word),
                 path_sep=>'/'};
     }
@@ -70,7 +70,7 @@ sub _default_completion {
     # treated like [AB]*.
     require String::Wildcard::Bash;
     if (String::Wildcard::Bash::contains_wildcard($word)) {
-        $log->tracef("completing with wildcard glob");
+        $log->tracef("[comp][compgl] completing with wildcard glob, glob=<%s>", "$word*");
         {
             my $compres = [glob("$word*")];
             last unless @$compres;
@@ -81,7 +81,7 @@ sub _default_completion {
         }
         # if empty, fallback to searching file
     }
-    $log->tracef("completing with file");
+    $log->tracef("[comp][compgl] completing with file, file=<%s>", $word);
     return {words=>Complete::Util::complete_file(word=>$word),
             path_sep=>'/'};
 }
@@ -259,6 +259,10 @@ sub complete_cli_arg {
     my $comp = $comp0 // \&_default_completion;
     my $extras = $args{extras} // {};
 
+    $log->tracef('[comp][compgl] entering %s::%s(), words=%s, cword=%d, word=<%s>, extras=%s',
+                 __PACKAGE__, "complete_cli_arg", \@words, $cword,
+                 $words[$cword], $extras); # XXX use __SUB__
+
     # parse all options first & supply default completion routine
     my %opts;
     for my $ospec (keys %$gospec) {
@@ -455,8 +459,11 @@ sub complete_cli_arg {
             }
         }
         #use DD; dd \@o;
-        push @res, @{ Complete::Util::complete_array_elem(
-            array => \@o, word => $word) };
+        my $compres = Complete::Util::complete_array_elem(
+            array => \@o, word => $word);
+        $log->tracef('[comp][compgl] adding result from option names, '.
+                         'matching options=%s', $compres);
+        push @res, @$compres;
         return {words=>\@res, escmode=>'option'}
             if !exists($exp->{optval}) && !exists($exp->{arg});
     }
@@ -472,9 +479,15 @@ sub complete_cli_arg {
             word=>$word, opt=>$opt, ospec=>$opthash->{ospec},
             argpos=>undef, nth=>$exp->{nth}, seen_opts=>\%seen_opts,
         );
+        $log->tracef('[comp][compgl] invoking \'completion\' routine '.
+                         'to complete option value, option=<%s>', $opt);
         my $compres = $comp->(%compargs);
+        $log->tracef('[comp][compgl] adding result from \'completion\' '.
+                         'routine: %s', $compres);
         if (!defined $compres) {
             $compres = _default_completion(%compargs);
+            $log->tracef('[comp][compgl] adding result from default '.
+                             'completion routine: %s', $compres);
         }
         if (ref($compres) eq 'ARRAY') {
             push @res, @$compres;
@@ -493,9 +506,13 @@ sub complete_cli_arg {
             word=>$word, opt=>undef, ospec=>undef,
             argpos=>$exp->{argpos}, seen_opts=>\%seen_opts,
         );
+        $log->tracef('[comp][compgl] invoking \'completion\' routine '.
+                         'to complete argument');
         my $compres = $comp->(%compargs);
         if (!defined $compres) {
             $compres = _default_completion(%compargs);
+            $log->tracef('[comp][compgl] adding result from default '.
+                             'completion routine: %s', $compres);
         }
         if (ref($compres) eq 'ARRAY') {
             push @res, @$compres;
@@ -523,7 +540,7 @@ Complete::Getopt::Long - Complete command-line argument using Getopt::Long speci
 
 =head1 VERSION
 
-This document describes version 0.21 of Complete::Getopt::Long (from Perl distribution Complete-Getopt-Long), released on 2014-12-25.
+This document describes version 0.22 of Complete::Getopt::Long (from Perl distribution Complete-Getopt-Long), released on 2014-12-26.
 
 =head1 SYNOPSIS
 
@@ -681,7 +698,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Complete-G
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/sharyanto/perl-Complete-Getopt-Long>.
+Source repository is at L<https://github.com/perlancar/perl-Complete-Getopt-Long>.
 
 =head1 BUGS
 
